(ns minibaldur.ecs
  (:require [clojure.string :as str]))

(defonce next-entity (atom 1))
(defonce ^:dynamic *components* (atom {}))

(defmacro defcomponent
  [comp-name]
  `(do
     (swap! *components* assoc ~(keyword comp-name) {})
     (defn ~comp-name
       ([entity#] (get-in @*components* [~(keyword comp-name) entity#]))
       ([entity# value#] (swap! *components* assoc-in [~(keyword comp-name) entity#] value#)))))

(defmacro defentity
  [& components]
  (let [entity (gensym)]
    `(let [~entity (first (swap-vals! next-entity inc))]
       ~@(for [[comp-name comp-value] components]
           `(~comp-name ~entity ~comp-value)))))

(defn query-e
  "Return known entitys matching a query"
  [query]
  (if (keyword? query)
    (some-> (query @*components*) keys)
    (loop [ents (query-e (first query))
           fils (next query)]
      (if (nil? fils)
        ents
        (recur (filter (set ents) (query-e (first fils))) (next fils))))))

(defn run-e
  "Run system on all entities matching query"
  [f query & args]
  (doseq [entity (query-e query)]
    (apply f entity args)))

(defn find-by
  [component value]
  (some #(and (= value (val %)) (key %))
        (@*components* component)))
