(ns minibaldur.engine)

(defn- diceroll
  [x]
  (if-let [match (re-find #"(\d+)d(\d+)" x)]
    (map #(Integer/parseInt %) (next match))))

(defn- diceval
  [x]
  (if-let [[a b] (diceroll x)]
    (apply + (repeat a (inc (rand-int b))))
    (Integer/parseInt x)))

(defn roll
  "Takes a standard ttrpg roll specifier and returns a vector, starting with the sum, and then each roll in the order listed in the specifier"
  [specifier]
  (let [dies-and-ops (clojure.string/split specifier #"\s+")
        first-die-val (diceval (first dies-and-ops))
        first-die-roll? (diceroll (first dies-and-ops))]
    (loop [sum first-die-val
           rolls (if first-die-roll? [first-die-val] [])
           tail (next dies-and-ops)]
      (if-let [[op die] tail]
        (let [roll? (diceroll die)
              val (diceval die)]
          (case op
            "+" (recur (+ sum val) (if roll? (conj rolls val) rolls) (nnext tail))
            "-" (recur (- sum val) (if roll? (conj rolls val) rolls) (nnext tail))))
        (into [sum] rolls)))))

(defn roll-advantage
  "Rolls 2d20 and returns the highest. Takes an optional specifier for bonuses"
  ([] (max-key first (roll "1d20") (roll "1d20")))
  ([bonuses]
   (let [spec (str "1d20 + " bonuses)]
     (max-key first (roll spec) (roll spec)))))

(defn roll-disadvantage
  "Rolls 2d20 and returns the lowest. Takes an optional specifier for bonuses"
  ([] (min-key first (roll "1d20") (roll "1d20")))
    ([bonuses]
   (let [spec (str "1d20 + " bonuses)]
     (min-key first (roll spec) (roll spec)))))
